/*
** Copyright (c) Massachusetts Institute of Technology 1994, 1995, 1996.
**          All Rights Reserved.
**          Unpublished rights reserved under the copyright laws of
**          the United States.
**
** THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY EXPRESSED
** OR IMPLIED.  ANY USE IS AT YOUR OWN RISK.
**
** This code is distributed freely and may be used freely under the 
** following conditions:
**
**     1. This notice may not be removed or altered.
**
**     2. This code may not be re-distributed or modified
**        without permission from MIT (contact 
**        lclint-request@larch.lcs.mit.edu.)  
**
**        Modification and re-distribution are encouraged,
**        but we want to keep track of changes and
**        distribution sites.
*/
/*
** fileloc.h
*/

# ifndef FILELOC_H
# define FILELOC_H

/* typedef in forwardTypes */
typedef /*@only@*/ fileloc o_fileloc;

# include "fileId.h"

extern fileloc fileloc_update (/*@only@*/ fileloc old, fileloc new) 
   /*@modifies old@*/ ;

extern fileloc fileloc_create (fileId fid, int line, int col) /*@*/ ;
extern bool fileloc_isSystemFile (fileloc f1) /*@*/ ;
extern fileloc fileloc_createSpec (fileId fid, int line, int col) /*@*/ ;
extern fileloc fileloc_createLib (cstring ln) /*@*/ ;
extern fileloc fileloc_createRc (cstring ln) /*@*/ ;
extern fileloc fileloc_decColumn (fileloc f, int x) /*@*/ ;
extern void fileloc_subColumn (fileloc f, int x) /*@modifies f@*/ ;
extern fileloc fileloc_getBuiltin (void) /*@*/ ;
extern /*@observer@*/ fileloc fileloc_observeBuiltin (void) /*@*/ ;
extern fileloc fileloc_createBuiltin (void) /*@*/ ;
extern fileloc fileloc_createImport (cstring fname, int lineno) /*@*/ ;
extern bool fileloc_isSpecialFile (fileloc f) /*@*/ ;
extern bool fileloc_sameBaseFile (fileloc f1, fileloc f2) /*@*/ ;
extern /*@observer@*/ cstring fileloc_filename (fileloc f) /*@*/ ;
extern int fileloc_column (fileloc f) /*@*/ ;
extern cstring fileloc_unparse (fileloc f) /*@*/ ;
extern bool fileloc_sameFile (fileloc f1, fileloc f2) /*@*/ ;
extern void fileloc_free (/*@only@*/ fileloc f);
extern void fileloc_reallyFree (/*@only@*/ fileloc f);
extern int fileloc_lineno (fileloc f) /*@*/ ;
extern bool fileloc_equal (fileloc f1, fileloc f2) /*@*/ ;
extern bool fileloc_lessthan (fileloc f1, fileloc f2) /*@*/ ;
extern int fileloc_compare (fileloc f1, fileloc f2) /*@*/ ;
extern /*@observer@*/ cstring fileloc_getBase (fileloc f) /*@*/ ;
extern bool fileloc_isHeader (fileloc f) /*@*/ ;
extern bool fileloc_isSpec (fileloc f) /*@*/ ;
extern bool fileloc_isRealSpec (fileloc f) /*@*/ ;
extern fileloc fileloc_copy (fileloc f) /*@*/ ;
extern cstring fileloc_unparseDirect (fileloc fl) /*@*/ ;
extern bool fileloc_notAfter (fileloc f1, fileloc f2) /*@*/ ;
extern bool fileloc_almostSameFile (fileloc f1, fileloc f2) /*@*/ ;
extern fileloc fileloc_noColumn (fileloc f) /*@*/ ;
extern fileloc fileloc_createExternal (void) /*@*/ ;

extern bool fileloc_isExternal (/*@sef@*/ fileloc f) /*@*/;
# define fileloc_isExternal(f) \
     (fileloc_isDefined(f) && ((f)->kind == FL_EXTERNAL))

extern /*@falsenull@*/ bool fileloc_isDefined (/*@null@*/ fileloc f) /*@*/ ;
extern /*@truenull@*/ bool fileloc_isUndefined (/*@null@*/ fileloc f) /*@*/ ;
extern bool fileloc_isInvalid (/*@sef@*/ /*@null@*/ fileloc f) /*@*/ ;

/*@constant null fileloc fileloc_undefined; @*/
# define fileloc_undefined      ((fileloc)0)
# define fileloc_isDefined(f)   ((f) != fileloc_undefined)
# define fileloc_isUndefined(f) ((f) == fileloc_undefined)
# define fileloc_isInvalid(f)   (!(fileloc_isValid(f)))

extern bool fileloc_isLib (fileloc f) /*@*/ ;

extern fileloc fileloc_fromTok (ltoken t) /*@*/ ;

/*@constant int UNKNOWN_LINE; @*/
# define UNKNOWN_LINE (0)

/*@constant int UNKNOWN_COLUMN; @*/
# define UNKNOWN_COLUMN (0)

extern /*@unused@*/ bool fileloc_linenoDefined (/*@sef@*/ fileloc f) /*@*/ ;
extern /*@unused@*/ bool fileloc_columnDefined (/*@sef@*/ fileloc f) /*@*/ ;

# define fileloc_linenoDefined(f) \
  (fileloc_isValid (f) && (f)->lineno != UNKNOWN_LINE)

# define fileloc_columnDefined(f) \
  (fileloc_isValid (f) && (f)->column != UNKNOWN_COLUMN)

extern void fileloc_setColumnUndefined (/*@sef@*/ fileloc f) /*@modifies f@*/;
# define fileloc_setColumnUndefined(f) \
  (fileloc_isDefined(f) ? (f)->column = UNKNOWN_COLUMN : UNKNOWN_COLUMN)

extern /*@falsenull@*/ bool fileloc_isValid (/*@sef@*/ fileloc f);
# define fileloc_isValid(f) \
  (fileloc_isDefined(f) && ((f)->lineno >= 0))

typedef enum 
{ 
  FL_NORMAL, FL_SPEC, FL_LIB, FL_STDLIB, FL_STDHDR, FL_IMPORT, FL_BUILTIN,
  FL_PREPROC, FL_RC, FL_EXTERNAL
} flkind;

struct __fileloc
{
  flkind  kind;
  fileId  fid;
  int     lineno;
  int     column;
} ;

extern bool fileloc_isImport (/*@sef@*/ fileloc fl);
# define fileloc_isImport(fl)  (fileloc_isDefined(fl) && (fl)->kind == FL_IMPORT)

extern bool fileloc_isPreproc (/*@sef@*/ fileloc fl);
# define fileloc_isPreproc(fl)  (fileloc_isDefined(fl) && (fl)->kind == FL_PREPROC)

extern void fileloc_setLineno (/*@sef@*/ fileloc f, int i) /*@modifies f@*/ ;
# define fileloc_setLineno(fl, i) \
      (fileloc_isDefined(fl) ? ((fl)->lineno = (i), (fl)->column = 1) : (i))

extern void fileloc_nextLine (/*@sef@*/ fileloc f) /*@modifies f@*/ ;
# define fileloc_nextLine(fl) \
      (fileloc_isDefined(fl) ? ((fl)->lineno++, (fl)->column = 1) : 0)

extern void fileloc_addLine (/*@sef@*/ fileloc f, int i) /*@modifies f@*/ ;
# define fileloc_addLine(fl, i) \
      (fileloc_isDefined(fl) ? ((fl)->lineno += (i), (fl)->column = 1) : (i))

extern fileId fileloc_fileId (/*@sef@*/ fileloc fl) /*@*/ ;
# define fileloc_fileId(fl) \
      (fileloc_isDefined (fl) ? (fl)->fid : fileId_invalid)

extern void fileloc_setColumn (/*@sef@*/ fileloc fl, int i) /*@modifies fl@*/ ;
# define fileloc_setColumn(fl, i) \
      (fileloc_isDefined (fl) ? ((fl)->column = (i)) : (i))

extern void fileloc_addColumn (/*@sef@*/ fileloc fl, int i) /*@modifies fl@*/ ;
# define fileloc_addColumn(fl, i) \
      (fileloc_isDefined (fl) ? ((fl)->column += (i)) : (i))

extern void fileloc_incColumn (/*@sef@*/ fileloc fl) /*@modifies fl@*/ ;
# define fileloc_incColumn(fl) \
      (fileloc_isDefined (fl) ? ((fl)->column++) : 0)

extern bool fileloc_isBuiltin (/*@sef@*/ fileloc fl) /*@*/ ;
# define fileloc_isBuiltin(fl) \
   (fileloc_isDefined (fl) && ((fl)->kind == FL_BUILTIN))

extern fileloc fileloc_updateFileId (/*@only@*/ fileloc old, fileId s) /*@*/ ;
extern fileloc fileloc_makePreproc (fileloc loc) /*@*/ ;
extern fileloc fileloc_makePreprocPrevious (fileloc loc) /*@*/ ;
extern bool fileloc_isStandardLibrary (fileloc f) /*@*/ ;
extern bool fileloc_isStandardLib (fileloc f) /*@*/ ;
extern cstring fileloc_unparseFilename (fileloc f) /*@*/ ;
extern bool fileloc_withinLines (fileloc f1, fileloc f2, int n) /*@*/ ;
extern bool fileloc_isUser (fileloc f) /*@*/ ;
extern cstring fileloc_lineMarker (fileloc f) /*@*/ ;
extern cstring fileloc_previousLineMarker (fileloc f) /*@*/ ;
extern bool fileloc_sameModule (fileloc f1, fileloc f2) /*@*/ ;

# else
# error "Multiple include"
# endif






