/*
** Copyright (c) Massachusetts Institute of Technology 1994, 1995, 1996.
**          All Rights Reserved.
**          Unpublished rights reserved under the copyright laws of
**          the United States.
**
** THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY EXPRESSED
** OR IMPLIED.  ANY USE IS AT YOUR OWN RISK.
**
** This code is distributed freely and may be used freely under the 
** following conditions:
**
**     1. This notice may not be removed or altered.
**
**     2. This code may not be re-distributed or modified
**        without permission from MIT (contact 
**        lclint-request@larch.lcs.mit.edu.)  
**
**        Modification and re-distribution are encouraged,
**        but we want to keep track of changes and
**        distribution sites.
*/
# ifndef SYSTEM_CONSTANTS_H
# define SYSTEM_CONSTANTS_H

/* exit codes */

/*@constant int LLSUCCESS; @*/
# define LLSUCCESS   EXIT_SUCCESS

/*@constant int LLFAILURE; @*/
# define LLFAILURE   EXIT_FAILURE

/*@constant int LLGIVEUP; @*/
# define LLGIVEUP    EXIT_FAILURE

/*@constant int LLINTERRUPT; @*/
# define LLINTERRUPT EXIT_FAILURE

/*@constant int DEFAULTMAXMODS; @*/
# define DEFAULTMAXMODS 32

/*
** number of parse errors before quiting
*/

/*@constant int GIVEUPPARSE; @*/
# define GIVEUPPARSE  12 

/*
** how many recursive searches to do before bailing!
*/

/*@constant int MAXDEPTH; @*/
# define MAXDEPTH 20

/*
** This limits the number of recursive calls to aliasTable_canAliasAux.
** It is necessary in the presence on recursive nested data structures.
** Note that we don't want to simply detect recursion and quit, since
** aliasing may be apparent only at the deeper level.  It would be better
** to do something in the alias table to confirm that there are no aliases
** recorded below a given depth, but in practice this is much easier!
*/

/*@constant int ALIASSEARCHLIMIT; @*/
# define ALIASSEARCHLIMIT 15

/*
** Note /lib/cpp and gcc -E do NOT process all code the
** same way.  One common difference is concatenation and
** processing of comments.  This can be overridden using
** the environment variable:
**
**    setenv LCLINT_CPPCMD 'gcc -E' 
**
*/

/*
** The settings for BIG_MEMORY were developed on machines
** having 162M and 192M, so by BIG we mean at least a 
** decent size memory for a engineering workstation.
** 
** There are trade-offs between memory usage and execution-time.
** These are not well understood, and will depend a lot on
** the specific machine's paging characteristics, and the 
** type of code being checked.
*/

/*
** This one's compilcated.  You probably don't want to change
** it.
**
** lclint maintains a table of ctype's (its representation of
** C types) and represents types internally as an index into
** this table.  This greatly improves performance of type
** checking and saves memory.  For simple derived types, it
** is easy to find if the type is already in the table.  For
** more complicated types (e.g., functions and structures),
** we check if a type is already in the table by looking 
** backwards for matches.  The last DEFAULT_OPTLEVEL types
** entered into the table are checked.  Increasing this value
** may improve memory performance, but cost in execution time.
**
*/

/*@constant int DEFAULT_OPTLEVEL; @*/
# define DEFAULT_OPTLEVEL 15

/*
** There are many *List and *Set types used by lclint (all
** generated automatically using a template and script).  
** For each, the BASESIZE is the number of entries the
** structure grows by each time it runs out of space.
** Depending on expected usage, each List/Set defines
** its BASESIZE to one of these three values.
*/

/*@constant int SMALLBASESIZE; @*/
# define SMALLBASESIZE   4

/*@constant int MIDBASESIZE; @*/
# define MIDBASESIZE     8

/*@constant int LARGEBASESIZE; @*/
# define LARGEBASESIZE  16

/*@constant int BIGBASESIZE; @*/
# define BIGBASESIZE   256

/*@constant int HUGEBASESIZE; @*/
# define HUGEBASESIZE  512

/*
** Size of hash table for file names.
*/

/*@constant int FTHASHSIZE; @*/
# define FTHASHSIZE 273

/*
** BASESIZE for C environment frames.
*/

/*@constant int CBASESIZE; @*/
# define CBASESIZE 4

/*
** The global environment (across files) is stored in
** a hash table.
*/

/*@constant int CGLOBBASESIZE; @*/
# define CGLOBBASESIZE 1024

/*@constant int CGLOBHASHSIZE; @*/
# define CGLOBHASHSIZE 1795

/*
** Symbol table for lcl
*/

/*@constant int LLHASHSIZE; @*/
# define LLHASHSIZE 1279   

/* lclsyntable */
/*@constant unsigned long int INITSYNTABLE; @*/
# define INITSYNTABLE   1024

/*@constant double DELTASYNTABLE; @*/
# define DELTASYNTABLE  2.0

/* syntable */
/*@constant int SYNTABLE_BASESIZE; @*/
# define SYNTABLE_BASESIZE 256   

/* lcltokentable */
/*@constant unsigned long int INITTOKENTABLE; @*/
# define INITTOKENTABLE   1024

/*@constant double DELTATOKENTABLE; @*/
# define DELTATOKENTABLE  2.0

/* string2sym */

/*@constant int INITCHARSTRING; @*/
# define INITCHARSTRING    8192 

/*@constant int DELTACHARSTRING; @*/
# define DELTACHARSTRING   2  

/*@constant int INITSTRINGENTRY; @*/
# define INITSTRINGENTRY   1024

/*@constant int DELTASTRINGENTRY; @*/
# define DELTASTRINGENTRY  2  

/*@constant int HASHSIZE; @*/
# define HASHSIZE          1024

/*@constant int HASHMASK; @*/
# define HASHMASK          (HASHSIZE-1)	/* HASHSIZE must be 2^n     */

# else
# error "Multiple include"
# endif






